@@ -26,8 +26,6 @@
 #include <time.h>
 
 #include <glib.h>
-#include <libgnomevfs/gnome-vfs-types.h>
-#include <libgnomevfs/gnome-vfs-mime.h>
 
 #include "file-data.h"
 #include "file-utils.h"
@@ -35,19 +33,20 @@
 #include "fr-command.h"
 #include "fr-command-7z.h"
 
-static void fr_command_7z_class_init  (FRCommand7zClass *class);
-static void fr_command_7z_init        (FRCommand         *afile);
+static void fr_command_7z_class_init  (FrCommand7zClass *class);
+static void fr_command_7z_init        (FrCommand         *afile);
 static void fr_command_7z_finalize    (GObject           *object);
 
 /* Parent Class */
 
-static FRCommandClass *parent_class = NULL;
+static FrCommandClass *parent_class = NULL;
 
 
 /* -- list -- */
 
+
 static time_t
-mktime_from_string (char *date_s, 
+mktime_from_string (char *date_s,
 		    char *time_s)
 {
 	struct tm   tm = {0, };
@@ -81,12 +80,12 @@ mktime_from_string (char *date_s,
 	return mktime (&tm);
 }
 
-
+/*
 static void
 change_to_unix_dir_separator (char *path)
 {
 	char *c;
-	
+
 	for (c = path; *c != 0; c++)
 		if ((*c == '\\') && (*(c+1) != '\\'))
 			*c = '/';
@@ -98,10 +97,10 @@ to_dos (const char *path)
 {
 	return str_substitute (path, "/", "\\\\");
 }
-
+*/
 
 static int
-str_rfind (const char *str, 
+str_rfind (const char *str,
 	   int         c)
 {
 	char *tmp = strrchr (str, c);
@@ -110,11 +109,11 @@ str_rfind (const char *str,
 
 
 static void
-list__process_line (char     *line, 
+list__process_line (char     *line,
 		    gpointer  data)
 {
-	FRCommand   *comm = FR_COMMAND (data);
-	FRCommand7z *p7z_comm = FR_COMMAND_7Z (comm);
+	FrCommand   *comm = FR_COMMAND (data);
+	FrCommand7z *p7z_comm = FR_COMMAND_7Z (comm);
 	FileData    *fdata;
 	char        *name_field;
 	char       **fields;
@@ -144,53 +143,72 @@ list__process_line (char     *line,
 		return;
 	}
 
-	if (fields[2][0] == 'D') { /* skip directories */
-		g_strfreev (fields);
-		file_data_free (fdata);
-		return;
-	}
-
 	fdata->size = g_ascii_strtoull (fields[3], NULL, 10);
-	fdata->modified = mktime_from_string (fields[0], fields[1]); 
+	fdata->modified = mktime_from_string (fields[0], fields[1]);
+	fdata->dir = fields[2][0] == 'D';
 	g_strfreev (fields);
 
 	name_field = g_strdup (line + p7z_comm->name_index);
-
-	change_to_unix_dir_separator (name_field);
-	
-	if (*name_field == '/') {
-		fdata->full_path = g_strdup (name_field);
-		fdata->original_path = fdata->full_path;
-	} else {
-		fdata->full_path = g_strconcat ("/", name_field, NULL);
-		fdata->original_path = fdata->full_path + 1;
-	}
+	fdata->free_original_path = TRUE;
+	fdata->original_path = g_strdup (name_field);
+	fdata->full_path = g_strconcat ((fdata->original_path[0] != '/') ? "/" : "", 
+					fdata->original_path,
+					(fdata->dir && (fdata->original_path[strlen (fdata->original_path - 1)] != '/')) ? "/" : "",
+					NULL);
 	g_free (name_field);
 
 	fdata->link = NULL;
-		
-	fdata->name = g_strdup (file_name_from_path (fdata->full_path));
-	fdata->path = remove_level_from_path (fdata->full_path);
-	fdata->type = gnome_vfs_mime_type_from_name_or_default (fdata->name, GNOME_VFS_MIME_TYPE_UNKNOWN);
 
-	comm->file_list = g_list_prepend (comm->file_list, fdata);
+	if (fdata->dir)
+		fdata->name = dir_name_from_path (fdata->full_path);
+	else
+		fdata->name = g_strdup (file_name_from_path (fdata->full_path));
+	fdata->path = remove_level_from_path (fdata->full_path);
+
+	fr_command_add_file (comm, fdata);
 }
 
 
 static void
-fr_command_7z_begin_command (FRCommand *comm)
+fr_command_7z_begin_command (FrCommand *comm)
 {
 	if (is_program_in_path ("7za"))
 		fr_process_begin_command (comm->process, "7za");
+	else if (is_program_in_path ("7zr"))
+		fr_process_begin_command (comm->process, "7zr");
 	else
 		fr_process_begin_command (comm->process, "7z");
 }
 
 
 static void
-fr_command_7z_list (FRCommand *comm)
+add_password_arg (FrCommand     *comm,
+		  const char    *password,
+		  gboolean       always_specify)
 {
-	fr_process_set_out_line_func (FR_COMMAND (comm)->process, 
+	if (always_specify || ((password != NULL) && (*password != 0))) {
+		char *arg;
+		char *e_password;
+
+		e_password = escape_str (password, "\"*?[]'`()$!;");
+		if (e_password != NULL) {
+			arg = g_strconcat ("-p\"", e_password, "\"", NULL);
+			g_free (e_password);
+		} 
+		else
+			arg = g_strdup ("-p\"\"");
+
+		fr_process_add_arg (comm->process, arg);
+		g_free (arg);
+	}
+}
+
+
+static void
+fr_command_7z_list (FrCommand  *comm,
+		    const char *password)
+{
+	fr_process_set_out_line_func (FR_COMMAND (comm)->process,
 				      list__process_line,
 				      comm);
 
@@ -198,26 +216,30 @@ fr_command_7z_list (FRCommand *comm)
 	fr_process_add_arg (comm->process, "l");
 	fr_process_add_arg (comm->process, "-bd");
 	fr_process_add_arg (comm->process, "-y");
-	fr_process_add_arg (comm->process, comm->e_filename);
+	add_password_arg (comm, password, FALSE);
+	fr_process_add_arg (comm->process, comm->filename);
 	fr_process_end_command (comm->process);
 	fr_process_start (comm->process);
 }
 
 
 static void
-fr_command_7z_add (FRCommand     *comm,
+fr_command_7z_add (FrCommand     *comm,
 		   GList         *file_list,
 		   const char    *base_dir,
 		   gboolean       update,
+		   gboolean       recursive,
 		   const char    *password,
-		   FRCompression  compression)
+		   FrCompression  compression)
 {
 	GList *scan;
 
 	fr_command_7z_begin_command (comm);
 
-	if (base_dir != NULL) 
+	if (base_dir != NULL) {
 		fr_process_set_working_dir (comm->process, base_dir);
+		fr_process_add_arg_concat (comm->process, "-w", base_dir, NULL);
+	}
 
 	if (update)
 		fr_process_add_arg (comm->process, "u");
@@ -226,6 +248,8 @@ fr_command_7z_add (FRCommand     *comm,
 
 	fr_process_add_arg (comm->process, "-bd");
 	fr_process_add_arg (comm->process, "-y");
+	fr_process_add_arg (comm->process, "-l");
+	add_password_arg (comm, password, FALSE);
 
 	switch (compression) {
 	case FR_COMPRESSION_VERY_FAST:
@@ -238,12 +262,11 @@ fr_command_7z_add (FRCommand     *comm,
 		fr_process_add_arg (comm->process, "-mx=7"); break;
 	}
 
-	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_add_arg (comm->process, comm->filename);
 
 	for (scan = file_list; scan; scan = scan->next) {
-		char *filename = to_dos (scan->data);
+		char *filename = scan->data;
 		fr_process_add_arg (comm->process, filename);
-		g_free (filename);
 	}
 
 	fr_process_end_command (comm->process);
@@ -251,7 +274,7 @@ fr_command_7z_add (FRCommand     *comm,
 
 
 static void
-fr_command_7z_delete (FRCommand *comm,
+fr_command_7z_delete (FrCommand *comm,
 		      GList     *file_list)
 {
 	GList *scan;
@@ -261,20 +284,17 @@ fr_command_7z_delete (FRCommand *comm,
 	fr_process_add_arg (comm->process, "-bd");
 	fr_process_add_arg (comm->process, "-y");
 
-	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_add_arg (comm->process, comm->filename);
 
-	for (scan = file_list; scan; scan = scan->next) {
-		char *filename = to_dos (scan->data);
-		fr_process_add_arg (comm->process, filename);
-		g_free (filename);
-	}
+	for (scan = file_list; scan; scan = scan->next) 
+		fr_process_add_arg (comm->process, scan->data);
 
 	fr_process_end_command (comm->process);
 }
 
 
 static void
-fr_command_7z_extract (FRCommand  *comm,
+fr_command_7z_extract (FrCommand  *comm,
 		       GList      *file_list,
 		       const char *dest_dir,
 		       gboolean    overwrite,
@@ -293,82 +313,104 @@ fr_command_7z_extract (FRCommand  *comm,
 
 	fr_process_add_arg (comm->process, "-bd");
 	fr_process_add_arg (comm->process, "-y");
+	add_password_arg (comm, password, FALSE);
 	
-	if (dest_dir != NULL) {
-		char *e_dest_dir = fr_command_escape (comm, dest_dir);
-		char *opt = g_strconcat ("-o", e_dest_dir, NULL);
-		fr_process_add_arg (comm->process, opt);
-		g_free (opt);
-		g_free (e_dest_dir);
-	}
+	if (dest_dir != NULL) 
+		fr_process_add_arg_concat (comm->process, "-o", dest_dir, NULL);
 
-	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_add_arg (comm->process, comm->filename);
 
-	for (scan = file_list; scan; scan = scan->next) {
-		char *filename = to_dos (scan->data);
-		fr_process_add_arg (comm->process, filename);
-		g_free (filename);
-	}
+	for (scan = file_list; scan; scan = scan->next) 
+		fr_process_add_arg (comm->process, scan->data);
 
 	fr_process_end_command (comm->process);
 }
 
 
 static void
-fr_command_7z_test (FRCommand   *comm,
+fr_command_7z_test (FrCommand   *comm,
 		    const char  *password)
 {
 	fr_command_7z_begin_command (comm);
 	fr_process_add_arg (comm->process, "t");
 	fr_process_add_arg (comm->process, "-bd");
 	fr_process_add_arg (comm->process, "-y");
-	fr_process_add_arg (comm->process, comm->e_filename);
+	add_password_arg (comm, password, FALSE);
+	fr_process_add_arg (comm->process, comm->filename);
 	fr_process_end_command (comm->process);
 }
 
 
-static void 
-fr_command_7z_class_init (FRCommand7zClass *class)
+static void
+fr_command_7z_handle_error (FrCommand   *comm,
+			    FrProcError *error)
 {
-        GObjectClass   *gobject_class = G_OBJECT_CLASS (class);
-        FRCommandClass *afc;
+	if (error->type == FR_PROC_ERROR_COMMAND_ERROR) {
+		if (error->status <= 1)
+			error->type = FR_PROC_ERROR_NONE;
+	}
+}
 
-        parent_class = g_type_class_peek_parent (class);
-	afc = (FRCommandClass*) class;
+
+static void
+fr_command_7z_set_mime_type (FrCommand  *comm,
+			     const char *mime_type)
+{
+	FR_COMMAND_CLASS (parent_class)->set_mime_type (comm, mime_type);
+	
+	comm->capabilities |= FR_COMMAND_CAP_ARCHIVE_MANY_FILES;
+	if (is_program_in_path ("7za")) 
+		comm->capabilities |= FR_COMMAND_CAP_READ_WRITE;
+	else if (is_program_in_path ("7zr")) 
+		comm->capabilities |= FR_COMMAND_CAP_READ_WRITE;
+	else if (is_program_in_path ("7z")) 
+		comm->capabilities |= FR_COMMAND_CAP_READ_WRITE;
+}
+
+
+static void
+fr_command_7z_class_init (FrCommand7zClass *class)
+{
+	GObjectClass   *gobject_class = G_OBJECT_CLASS (class);
+	FrCommandClass *afc;
+
+	parent_class = g_type_class_peek_parent (class);
+	afc = (FrCommandClass*) class;
 
 	gobject_class->finalize = fr_command_7z_finalize;
 
-        afc->list           = fr_command_7z_list;
+	afc->list           = fr_command_7z_list;
 	afc->add            = fr_command_7z_add;
 	afc->delete         = fr_command_7z_delete;
 	afc->extract        = fr_command_7z_extract;
 	afc->test           = fr_command_7z_test;
+	afc->handle_error   = fr_command_7z_handle_error;
+	afc->set_mime_type  = fr_command_7z_set_mime_type;
 }
 
- 
-static void 
-fr_command_7z_init (FRCommand *comm)
-{
-	comm->file_type = FR_FILE_TYPE_7ZIP;
 
-	comm->propAddCanUpdate             = TRUE; 
-	comm->propAddCanReplace            = TRUE; 
+static void
+fr_command_7z_init (FrCommand *comm)
+{
+	comm->propAddCanUpdate             = TRUE;
+	comm->propAddCanReplace            = TRUE;
+	comm->propAddCanStoreFolders       = TRUE;
 	comm->propExtractCanAvoidOverwrite = FALSE;
 	comm->propExtractCanSkipOlder      = FALSE;
 	comm->propExtractCanJunkPaths      = TRUE;
-	comm->propPassword                 = FALSE;
+	comm->propPassword                 = TRUE;
 	comm->propTest                     = TRUE;
 }
 
 
-static void 
+static void
 fr_command_7z_finalize (GObject *object)
 {
-        g_return_if_fail (object != NULL);
-        g_return_if_fail (FR_IS_COMMAND_7Z (object));
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (FR_IS_COMMAND_7Z (object));
 
 	/* Chain up */
-        if (G_OBJECT_CLASS (parent_class)->finalize)
+	if (G_OBJECT_CLASS (parent_class)->finalize)
 		G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -376,17 +418,17 @@ fr_command_7z_finalize (GObject *object)
 GType
 fr_command_7z_get_type ()
 {
-        static GType type = 0;
+	static GType type = 0;
 
-        if (! type) {
-                GTypeInfo type_info = {
-			sizeof (FRCommand7zClass),
+	if (! type) {
+		GTypeInfo type_info = {
+			sizeof (FrCommand7zClass),
 			NULL,
 			NULL,
 			(GClassInitFunc) fr_command_7z_class_init,
 			NULL,
 			NULL,
-			sizeof (FRCommand7z),
+			sizeof (FrCommand7z),
 			0,
 			(GInstanceInitFunc) fr_command_7z_init
 		};
@@ -395,25 +437,7 @@ fr_command_7z_get_type ()
 					       "FRCommand7z",
 					       &type_info,
 					       0);
-        }
-
-        return type;
-}
-
-
-FRCommand *
-fr_command_7z_new (FRProcess  *process,
-		   const char *filename)
-{
-	FRCommand *comm;
-
-	if ((!is_program_in_path("7za")) &&
-	    (!is_program_in_path("7z"))) {
-		return NULL;
 	}
 
-	comm = FR_COMMAND (g_object_new (FR_TYPE_COMMAND_7Z, NULL));
-	fr_command_construct (comm, process, filename);
-
-	return comm;
+	return type;
 }
