@@ -35,9 +35,6 @@
 #include "fr-command.h"
 #include "fr-command-cfile.h"
 
-static void fr_command_cfile_class_init  (FrCommandCFileClass *class);
-static void fr_command_cfile_init        (FrCommand           *afile);
-static void fr_command_cfile_finalize    (GObject             *object);
 
 /* Parent Class */
 
@@ -46,17 +43,17 @@ static FrCommandClass *parent_class = NULL;
 
 static char *
 get_uncompressed_name_from_archive (FrCommand  *comm, 
-				    const char *e_filename)
+				    const char *archive)
 {
 	int      fd;
 	char     buffer[11];
 	char    *filename = NULL;
 	GString *str = NULL;
 
-	if (is_mime_type (comm->mime_type, "application/x-gzip"))
+	if (! is_mime_type (comm->mime_type, "application/x-gzip"))
 		return NULL;
 	
-	fd = open (e_filename, O_RDONLY);
+	fd = open (archive, O_RDONLY);
 	if (fd < 0) 
 		return NULL;
 	
@@ -159,7 +156,7 @@ fr_command_cfile_list (FrCommand  *comm,
 		fr_process_begin_command (comm->process, "gzip");
 		fr_process_add_arg (comm->process, "-l");
 		fr_process_add_arg (comm->process, "-q");
-		fr_process_add_arg (comm->process, comm->e_filename);
+		fr_process_add_arg (comm->process, comm->filename);
 		fr_process_end_command (comm->process);
 		fr_process_start (comm->process);
 	} 
@@ -181,7 +178,7 @@ fr_command_cfile_list (FrCommand  *comm,
 
 		fdata->original_path = fdata->full_path + 1;
 		fdata->link = NULL;
-		fdata->size = get_file_size (comm->filename);
+		fdata->size = get_file_size_for_path (comm->filename);
 		fdata->modified = get_file_mtime_for_path (comm->filename);
 		
 		fdata->name = g_strdup (file_name_from_path (fdata->full_path));
@@ -207,27 +204,23 @@ fr_command_cfile_add (FrCommand     *comm,
 		      GList         *file_list,
 		      const char    *base_dir,
 		      gboolean       update,
+		      gboolean       recursive,
 		      const char    *password,
 		      FrCompression  compression)
 {
 	const char *filename;
 	char       *temp_dir;
-	char       *e_temp_dir;
 	char       *temp_file;
+	char       *compressed_filename;
 
 	if ((file_list == NULL) || (file_list->data == NULL))
 		return;
 
-	/* create a temp dir. */
-
-	temp_dir = get_temp_work_dir ();
-	e_temp_dir = fr_command_escape (comm, temp_dir);
-
 	/* copy file to the temp dir */
 
+	temp_dir = get_temp_work_dir ();
 	filename = file_list->data;
-
-	temp_file = g_strconcat (e_temp_dir, "/", filename, NULL);
+	temp_file = g_strconcat (temp_dir, "/", filename, NULL);
 
 	fr_process_begin_command (comm->process, "cp");
 	fr_process_set_working_dir (comm->process, base_dir);
@@ -245,6 +238,7 @@ fr_command_cfile_add (FrCommand     *comm,
 		fr_process_add_arg (comm->process, "--");
 		fr_process_add_arg (comm->process, filename);
 		fr_process_end_command (comm->process);
+		compressed_filename = g_strconcat (filename, ".gz", NULL);
 	}
 	else if (is_mime_type (comm->mime_type, "application/x-bzip")) {
 		fr_process_begin_command (comm->process, "bzip2");
@@ -252,14 +246,15 @@ fr_command_cfile_add (FrCommand     *comm,
 		fr_process_add_arg (comm->process, "--");
 		fr_process_add_arg (comm->process, filename);
 		fr_process_end_command (comm->process);
+		compressed_filename = g_strconcat (filename, ".bz2", NULL);
 	}
-	else if (is_mime_type (comm->mime_type, "application/x-gzip")) {
+	else if (is_mime_type (comm->mime_type, "application/x-compress")) {
 		fr_process_begin_command (comm->process, "compress");
 		fr_process_set_working_dir (comm->process, temp_dir);
 		fr_process_add_arg (comm->process, "-f");
-		fr_process_add_arg (comm->process, "--");
 		fr_process_add_arg (comm->process, filename);
 		fr_process_end_command (comm->process);
+		compressed_filename = g_strconcat (filename, ".Z", NULL);
 	}
 	else if (is_mime_type (comm->mime_type, "application/x-lzma")) {
 		fr_process_begin_command (comm->process, "lzma");
@@ -267,6 +262,7 @@ fr_command_cfile_add (FrCommand     *comm,
 		fr_process_add_arg (comm->process, "--");
 		fr_process_add_arg (comm->process, filename);
 		fr_process_end_command (comm->process);
+		compressed_filename = g_strconcat (filename, ".lzma", NULL);
 	}
 	else if (is_mime_type (comm->mime_type, "application/x-lzop")) {
 		fr_process_begin_command (comm->process, "lzop");
@@ -276,16 +272,17 @@ fr_command_cfile_add (FrCommand     *comm,
 		fr_process_add_arg (comm->process, "--");
 		fr_process_add_arg (comm->process, filename);
 		fr_process_end_command (comm->process);
+		compressed_filename = g_strconcat (filename, ".lzo", NULL);
 	}
 
       	/* copy compressed file to the dest dir */
 
 	fr_process_begin_command (comm->process, "cp");
-	fr_process_set_working_dir (comm->process, e_temp_dir);
+	fr_process_set_working_dir (comm->process, temp_dir);
 	fr_process_add_arg (comm->process, "-f");
 	fr_process_add_arg (comm->process, "--");
-	fr_process_add_arg (comm->process, "*");
-	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_add_arg (comm->process, compressed_filename);
+	fr_process_add_arg (comm->process, comm->filename);
 	fr_process_end_command (comm->process);
 
 	/* remove the temp dir */
@@ -294,11 +291,11 @@ fr_command_cfile_add (FrCommand     *comm,
 	fr_process_set_sticky (comm->process, TRUE);
 	fr_process_add_arg (comm->process, "-rf");
 	fr_process_add_arg (comm->process, "--");
-	fr_process_add_arg (comm->process, e_temp_dir);
+	fr_process_add_arg (comm->process, temp_dir);
 	fr_process_end_command (comm->process);
 
+	g_free (compressed_filename);
 	g_free (temp_file);
-	g_free (e_temp_dir);
 	g_free (temp_dir);
 }
 
@@ -321,32 +318,23 @@ fr_command_cfile_extract (FrCommand  *comm,
 			  const char *password)
 {
 	char *temp_dir;
-	char *e_temp_dir;
 	char *dest_file;
-	char *e_dest_file;
 	char *temp_file;
-	char *e_temp_file;
 	char *uncompr_file;
-	char *e_uncompr_file;
 	char *compr_file;
 
-	/* create a temp dir. */
-
-	temp_dir = get_temp_work_dir ();
-	e_temp_dir = fr_command_escape (comm, temp_dir);
-
 	/* copy file to the temp dir, remove the already existing file first */
 
+	temp_dir = get_temp_work_dir ();
 	temp_file = g_strconcat (temp_dir,
 				 "/",
 				 file_name_from_path (comm->filename),
 				 NULL);
-	e_temp_file = fr_command_escape (comm, temp_file);
 
 	fr_process_begin_command (comm->process, "cp");
 	fr_process_add_arg (comm->process, "-f");
-	fr_process_add_arg (comm->process, comm->e_filename);
-	fr_process_add_arg (comm->process, e_temp_file);
+	fr_process_add_arg (comm->process, comm->filename);
+	fr_process_add_arg (comm->process, temp_file);
 	fr_process_end_command (comm->process);
 
 	/* uncompress the file */
@@ -356,27 +344,27 @@ fr_command_cfile_extract (FrCommand  *comm,
 		fr_process_add_arg (comm->process, "-f");
 		fr_process_add_arg (comm->process, "-d");
 		fr_process_add_arg (comm->process, "-n");
-		fr_process_add_arg (comm->process, e_temp_file);
+		fr_process_add_arg (comm->process, temp_file);
 		fr_process_end_command (comm->process);
 	}
 	else if (is_mime_type (comm->mime_type, "application/x-bzip")) {
 		fr_process_begin_command (comm->process, "bzip2");
 		fr_process_add_arg (comm->process, "-f");
 		fr_process_add_arg (comm->process, "-d");
-		fr_process_add_arg (comm->process, e_temp_file);
+		fr_process_add_arg (comm->process, temp_file);
 		fr_process_end_command (comm->process);
 	}
-	else if (is_mime_type (comm->mime_type, "application/x-gzip")) {
+	else if (is_mime_type (comm->mime_type, "application/x-compress")) {
 		fr_process_begin_command (comm->process, "uncompress");
 		fr_process_add_arg (comm->process, "-f");
-		fr_process_add_arg (comm->process, e_temp_file);
+		fr_process_add_arg (comm->process, temp_file);
 		fr_process_end_command (comm->process);
 	}
 	else if (is_mime_type (comm->mime_type, "application/x-lzma")) {
 		fr_process_begin_command (comm->process, "lzma");
 		fr_process_add_arg (comm->process, "-f");
 		fr_process_add_arg (comm->process, "-d");
-		fr_process_add_arg (comm->process, e_temp_file);
+		fr_process_add_arg (comm->process, temp_file);
 		fr_process_end_command (comm->process);
 	}
 	else if (is_mime_type (comm->mime_type, "application/x-lzop")) {
@@ -385,14 +373,13 @@ fr_command_cfile_extract (FrCommand  *comm,
 		fr_process_add_arg (comm->process, "-d");
 		fr_process_add_arg (comm->process, "-fU");
 		fr_process_add_arg (comm->process, "--no-stdin");
-		fr_process_add_arg (comm->process, e_temp_file);
+		fr_process_add_arg (comm->process, temp_file);
 		fr_process_end_command (comm->process);
 	}
 
 	/* copy uncompress file to the dest dir */
 
 	uncompr_file = remove_extension_from_path (temp_file);
-        e_uncompr_file = fr_command_escape (comm, uncompr_file);
         
 	compr_file = get_uncompressed_name_from_archive (comm, comm->filename);
 	if (compr_file == NULL) 
@@ -401,14 +388,11 @@ fr_command_cfile_extract (FrCommand  *comm,
 				 "/",
 				 compr_file,
 				 NULL);
-	e_dest_file = fr_command_escape (comm, dest_file);
-	g_free (compr_file);
-	g_free (dest_file);
 
 	fr_process_begin_command (comm->process, "cp");
 	fr_process_add_arg (comm->process, "-f");
-	fr_process_add_arg (comm->process, e_uncompr_file);
-	fr_process_add_arg (comm->process, e_dest_file);
+	fr_process_add_arg (comm->process, uncompr_file);
+	fr_process_add_arg (comm->process, dest_file);
 	fr_process_end_command (comm->process);
 
 	/* remove the temp dir */
@@ -416,15 +400,13 @@ fr_command_cfile_extract (FrCommand  *comm,
 	fr_process_begin_command (comm->process, "rm");
 	fr_process_set_sticky (comm->process, TRUE);
 	fr_process_add_arg (comm->process, "-rf");
-	fr_process_add_arg (comm->process, e_temp_dir);
+	fr_process_add_arg (comm->process, temp_dir);
 	fr_process_end_command (comm->process);
 
-	g_free (e_dest_file);
+	g_free (dest_file);
+	g_free (compr_file);
 	g_free (uncompr_file);
-	g_free (e_uncompr_file);
 	g_free (temp_file);
-	g_free (e_temp_file);
-	g_free (e_temp_dir);
 	g_free (temp_dir);
 }
 
@@ -443,7 +425,7 @@ fr_command_cfile_set_mime_type (FrCommand  *comm,
 		if (is_program_in_path ("bzip2"))
 			comm->capabilities |= FR_COMMAND_CAP_READ_WRITE;
 	}
-	else if (is_mime_type (comm->mime_type, "application/x-gzip")) {
+	else if (is_mime_type (comm->mime_type, "application/x-compress")) {
 		if (is_program_in_path ("compress"))
 			comm->capabilities |= FR_COMMAND_CAP_WRITE;
 		if (is_program_in_path ("uncompress"))
@@ -460,6 +442,18 @@ fr_command_cfile_set_mime_type (FrCommand  *comm,
 }
 
 
+static void 
+fr_command_cfile_finalize (GObject *object)
+{
+        g_return_if_fail (object != NULL);
+        g_return_if_fail (FR_IS_COMMAND_CFILE (object));
+
+	/* Chain up */
+        if (G_OBJECT_CLASS (parent_class)->finalize)
+                G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+
 static void 
 fr_command_cfile_class_init (FrCommandCFileClass *class)
 {
@@ -482,8 +476,8 @@ fr_command_cfile_class_init (FrCommandCFileClass *class)
 static void 
 fr_command_cfile_init (FrCommand *comm)
 {
-	comm->propAddCanUpdate             = FALSE;
-	comm->propAddCanReplace            = FALSE; 
+	comm->propAddCanUpdate             = TRUE;
+	comm->propAddCanReplace            = TRUE; 
 	comm->propExtractCanAvoidOverwrite = FALSE;
 	comm->propExtractCanSkipOlder      = FALSE;
 	comm->propExtractCanJunkPaths      = FALSE;
@@ -492,22 +486,6 @@ fr_command_cfile_init (FrCommand *comm)
 }
 
 
-static void 
-fr_command_cfile_finalize (GObject *object)
-{
-	FrCommandCFile *comm_tar;
-
-        g_return_if_fail (object != NULL);
-        g_return_if_fail (FR_IS_COMMAND_CFILE (object));
-
-	comm_tar = FR_COMMAND_CFILE (object);
-
-	 /* Chain up */
-        if (G_OBJECT_CLASS (parent_class)->finalize)
-                G_OBJECT_CLASS (parent_class)->finalize (object);
-}
-
-
 GType
 fr_command_cfile_get_type ()
 {
