@@ -55,7 +55,7 @@ typedef enum {
 	FR_ACTION_COPYING_FILES_TO_REMOTE,    /* copying extracted files to a remote location */
 	FR_ACTION_CREATING_ARCHIVE,           /* creating a local archive */
 	FR_ACTION_SAVING_REMOTE_ARCHIVE       /* copying the archive to a remote location */
-} FRAction;
+} FrAction;
 
 #ifdef DEBUG
 extern char *action_names[];
@@ -65,39 +65,41 @@ struct _FrCommand
 {
 	GObject  __parent;
 
-	FRFileType  file_type;
-	GList      *file_list;       /* FileData elements */
-
+	GPtrArray  *files;           /* Array of FileData* */
+	
 	/*<protected>*/
-
+	
 	/* properties the command supports. */
+	
+	guint       propAddCanUpdate : 1;
+	guint       propAddCanReplace : 1;
+	guint       propAddCanStoreFolders : 1;
+	guint       propExtractCanAvoidOverwrite : 1;
+	guint       propExtractCanSkipOlder : 1;
+	guint       propExtractCanJunkPaths : 1;
+	guint       propPassword : 1;
+	guint       propTest : 1;
+	guint       propCanExtractAll : 1;
+	guint       propCanDeleteNonEmptyFolders : 1;
+	guint       propCanExtractNonEmptyFolders : 1;
 
-	guint propCanModify : 1;
-	guint propAddCanUpdate : 1;
-	guint propAddCanReplace : 1;
-	guint propAddCanStoreFolders : 1;
-	guint propExtractCanAvoidOverwrite : 1;
-	guint propExtractCanSkipOlder : 1;
-	guint propExtractCanJunkPaths : 1;
-	guint propPassword : 1;
-	guint propTest : 1;
-	guint propCanExtractAll : 1;
-
-	/* used by the progress signal */
-
-	int     n_file;
-	int     n_files;         /* used by the progress signal */
-
+	/* progress data */
+	
+	int         n_file;          
+	int         n_files;
+	
 	/*<private>*/
 
-	FrProcess *process;      /* the process object used to execute
-				  * commands. */
-	FRAction   action;       /* current action. */
-	char      *filename;     /* archive filename. */
-	char      *e_filename;   /* escaped archive filename. */
-
-	gboolean   fake_load;    /* if TRUE does nothing when the list
-				  * operation is invoked. */
+	FrProcess  *process;         /* the process object used to execute
+				      * commands. */
+	FrAction    action;          /* current action. */
+	char       *filename;        /* archive file path. */
+	char       *e_filename;      /* escaped archive filename. */
+	const char *mime_type;
+	FrCommandCaps capabilities;
+	
+	gboolean    fake_load;       /* if TRUE does nothing when the list
+				      * operation is invoked. */
 };
 
 struct _FrCommandClass
@@ -108,17 +110,15 @@ struct _FrCommandClass
 
 	void        (*list)           (FrCommand     *comm,
 				       const char    *password);
-
 	void        (*add)            (FrCommand     *comm,
 				       GList         *file_list,
 				       const char    *base_dir,
 				       gboolean       update,
+				       gboolean       recursive,
 				       const char    *password,
-				       FRCompression  compression); 
-
+				       FrCompression  compression); 
 	void        (*delete)         (FrCommand     *comm,
 				       GList         *file_list); 
-
 	void        (*extract)        (FrCommand     *comm,
 				       GList         *file_list,
 				       const char    *dest_dir,
@@ -126,59 +126,46 @@ struct _FrCommandClass
 				       gboolean       skip_older,
 				       gboolean       junk_paths,
 				       const char    *password);
-
 	void        (*test)           (FrCommand     *comm,
 				       const char    *password);
-
 	void        (*uncompress)     (FrCommand     *comm);
-
 	void        (*recompress)     (FrCommand     *comm,
-				       FRCompression  compression);
-
+				       FrCompression  compression);
 	void        (*handle_error)   (FrCommand     *comm,
-				       FRProcError   *error);
+				       FrProcError   *error);
 
-	char *      (*escape)         (FrCommand     *comm,
-				       const char    *str);
+	/*<protected virtual functions>*/
+	
+	void        (*set_mime_type)  (FrCommand     *comm,
+				       const char    *mime_type);
 
 	/*<signals>*/
 
 	void        (*start)          (FrCommand   *comm,
-				       FRAction     action); 
-
+				       FrAction     action); 
 	void        (*done)           (FrCommand   *comm,
-				       FRAction     action,
-				       FRProcError *error);
-
+				       FrAction     action,
+				       FrProcError *error);
 	void        (*progress)       (FrCommand   *comm,
 				       double       fraction);
-
 	void        (*message)        (FrCommand   *comm,
 				       const char  *msg);
 };
 
 GType          fr_command_get_type           (void);
-
-void           fr_command_construct          (FrCommand     *comm,
-					      FrProcess     *process,
-					      const char    *filename);
-
 void           fr_command_set_filename       (FrCommand     *comm,
 					      const char    *filename);
-
 void           fr_command_list               (FrCommand     *comm,
 					      const char    *password);
-
 void           fr_command_add                (FrCommand     *comm,
 					      GList         *file_list,
 					      const char    *base_dir,
 					      gboolean       update,
+					      gboolean       recursive,
 					      const char    *password,
-					      FRCompression  compression); 
-
+					      FrCompression  compression); 
 void           fr_command_delete             (FrCommand     *comm,
 					      GList         *file_list); 
-
 void           fr_command_extract            (FrCommand     *comm,
 					      GList         *file_list,
 					      const char    *dest_dir,
@@ -186,32 +173,30 @@ void           fr_command_extract            (FrCommand     *comm,
 					      gboolean       skip_older,
 					      gboolean       junk_paths,
 					      const char    *password);
-
 void           fr_command_test               (FrCommand     *comm,
 					      const char    *password);
-
 void           fr_command_uncompress         (FrCommand     *comm);
-
 void           fr_command_recompress         (FrCommand     *comm,
-					      FRCompression  compression);
-
-char *         fr_command_escape             (FrCommand     *comm,
-					      const char    *str);
+					      FrCompression  compression);
+gboolean       fr_command_is_capable_of      (FrCommand     *comm, 
+					      FrCommandCaps  capabilities);
 
 /* protected functions */
 
 void           fr_command_progress           (FrCommand     *comm,
 					      double         fraction);
-
 void           fr_command_message            (FrCommand     *comm,
 					      const char    *msg);
-
 void           fr_command_set_n_files        (FrCommand     *comm,
 					      int            n_files);
+void           fr_command_add_file           (FrCommand     *comm,
+					      FileData      *fdata);
+void           fr_command_set_mime_type      (FrCommand     *comm,
+					      const char    *mime_type);
 
 /* private functions */
 
 void           fr_command_handle_error       (FrCommand     *comm,
-					      FRProcError   *error);
+					      FrProcError   *error);
 
 #endif /* FR_COMMAND_H */
