@@ -129,7 +129,6 @@ void        fr_window_archive_add_with_wildcard (FrWindow      *window,
 						 const char    *base_dir,
 						 const char    *dest_dir,
 						 gboolean       update,
-						 gboolean       recursive,
 						 gboolean       follow_links);
 void        fr_window_archive_add_directory     (FrWindow      *window,
 						 const char    *directory,
@@ -137,20 +136,20 @@ void        fr_window_archive_add_directory     (FrWindow      *window,
 						 const char    *dest_dir,
 						 gboolean       update,
 						 const char    *password,
-						 FRCompression  compression);
+						 FrCompression  compression);
 void        fr_window_archive_add_items         (FrWindow      *window,
 						 GList         *dir_list,
 						 const char    *base_dir,
 						 const char    *dest_dir,
 						 gboolean       update,
 						 const char    *password,
-						 FRCompression  compression);
+						 FrCompression  compression);
 void        fr_window_archive_add_dropped_items (FrWindow      *window,
 						 GList         *item_list,
 						 gboolean       update);
 void        fr_window_archive_remove            (FrWindow      *window,
 						 GList         *file_list,
-						 FRCompression  compression);
+						 FrCompression  compression);
 void        fr_window_archive_extract           (FrWindow      *window,
 						 GList         *file_list,
 						 const char    *extract_to_dir,
@@ -170,9 +169,7 @@ void        fr_window_set_password              (FrWindow      *window,
 void        fr_window_set_password_for_paste    (FrWindow      *window,
 					         const char    *password);						 
 const char *fr_window_get_password              (FrWindow      *window);
-FRCompression fr_window_get_compression 	(FrWindow      *window);
-void        fr_window_view_folder_after_extract (FrWindow      *window,
-					 	 gboolean       view);
+FrCompression fr_window_get_compression 	(FrWindow      *window);
 
 /**/
 
@@ -186,7 +183,7 @@ void       fr_window_go_forward                 (FrWindow       *window);
 void       fr_window_current_folder_activated   (FrWindow       *window,
 						 gboolean        from_sidebar);
 void       fr_window_set_list_mode              (FrWindow       *window,
-						 FRWindowListMode  list_mode);
+						 FrWindowListMode  list_mode);
 
 /**/
 
@@ -245,7 +242,7 @@ void        fr_window_open_files_with_command     (FrWindow   *window,
 void        fr_window_open_files_with_application (FrWindow   *window,
 						   GList      *file_list,
 						   GAppInfo   *app);
-void        fr_window_update_files                (FrWindow   *window,
+gboolean    fr_window_update_files                (FrWindow   *window,
 						   GList      *file_list);			   
 void        fr_window_update_columns_visibility   (FrWindow   *window);
 void        fr_window_update_history_list         (FrWindow   *window);
@@ -297,10 +294,10 @@ void       fr_window_set_batch__extract_here      (FrWindow      *window,
 void       fr_window_set_batch__add               (FrWindow      *window,
 						   const char    *archive,
 						   GList         *file_list);
+void       fr_window_destroy_with_error_dialog    (FrWindow      *window);
 
 /**/
 
-void       fr_window_convert_data_free       (FrWindow         *window);
 gboolean   fr_window_file_list_drag_data_get (FrWindow         *window,
 					      GdkDragContext   *context,
 					      GtkSelectionData *selection_data,
