@@ -50,7 +50,6 @@
 #define PREF_ADD_COMPRESSION_LEVEL "/apps/file-roller/general/compression_level"
 #define PREF_MIGRATE_DIRECTORIES   "/apps/file-roller/general/migrate_directories"
 
-#define PREF_EXTRACT_VIEW_FOLDER      "/apps/file-roller/dialogs/extract/view_destination_folder"
 #define PREF_EXTRACT_OVERWRITE        "/apps/file-roller/dialogs/extract/overwrite"
 #define PREF_EXTRACT_SKIP_NEWER       "/apps/file-roller/dialogs/extract/skip_newer"
 #define PREF_EXTRACT_RECREATE_FOLDERS "/apps/file-roller/dialogs/extract/recreate_folders"
@@ -72,14 +71,14 @@
 #define PREF_DESKTOP_TOOLBAR_DETACHABLE "/desktop/gnome/interface/toolbar_detachable"
 #define PREF_NAUTILUS_CLICK_POLICY "/apps/nautilus/preferences/click_policy"
 
-FRWindowSortMethod  preferences_get_sort_method       (void);
-void                preferences_set_sort_method       (FRWindowSortMethod  i_value);
+FrWindowSortMethod  preferences_get_sort_method       (void);
+void                preferences_set_sort_method       (FrWindowSortMethod  i_value);
 GtkSortType         preferences_get_sort_type         (void);
 void                preferences_set_sort_type         (GtkSortType         i_value);
-FRWindowListMode    preferences_get_list_mode         (void);
-void                preferences_set_list_mode         (FRWindowListMode    i_value);
-FRCompression       preferences_get_compression_level (void);
-void                preferences_set_compression_level (FRCompression       i_value);
+FrWindowListMode    preferences_get_list_mode         (void);
+void                preferences_set_list_mode         (FrWindowListMode    i_value);
+FrCompression       preferences_get_compression_level (void);
+void                preferences_set_compression_level (FrCompression       i_value);
 void                pref_util_save_window_geometry    (GtkWindow          *window,
 						       const char         *dialog);
 void                pref_util_restore_window_geometry (GtkWindow          *window,
